use std::fs::File;
use std::io::{Read, Write};

use serde_derive::Deserialize;

use crate::result::{MBGError, MBGResult};

pub mod image_processors;
pub mod logger;

#[derive(Serialize, Default, Deserialize, Debug)]
pub struct Config {
    #[serde(default)]
    pub blender: Vec<u8>,
    #[serde(default = "reset_command_default")]
    pub reset_command: String,
    #[serde(default)]
    pub logger: logger::LoggerConfig,
    #[serde(default)]
    pub processors: Vec<image_processors::ProcessorParams>,
}

pub fn reset_command_default() -> String {
    String::from("nitrogen --restore")
}

impl Config {
    fn new() -> Self {
        Config {
            reset_command: reset_command_default(),
            ..Default::default()
        }
    }

    fn read_props() -> MBGResult<Self> {
        let home_dir = dirs::home_dir();
        if home_dir.is_none() {
            return Err(MBGError::ConfigErr(String::from(
                "Can't locate home directory",
            )));
        }
        let mut config_path = home_dir.unwrap();
        config_path.push(".mbg.toml");
        let mut conf_file = File::open(config_path)?;
        let mut contents = String::new();
        conf_file.read_to_string(&mut contents)?;
        toml::from_str::<Config>(contents.as_str()).map_err(MBGError::from)
    }

    pub fn get_props() -> Self {
        let props = Self::read_props();
        match props {
            Err(error) => {
                error!("Error while parsing property file. Consider updating ~/.mbg.toml");
                error!("{:?}", error);
                info!("Using default properties");
                Self::new()
            }
            Ok(props) => props,
        }
    }

    pub fn generate_config() -> MBGResult<()> {
        let home_dir = dirs::home_dir();
        if home_dir.is_none() {
            return Err(MBGError::ConfigErr(String::from(
                "Can't locate home directory",
            )));
        }
        let mut config_path = home_dir.unwrap();
        config_path.push(".mbg.toml");
        let mut conf_file = File::create(config_path)?;
        let mut default_conf = Self::new();
        default_conf
            .processors
            .push(image_processors::ProcessorParams::default());
        default_conf.blender.push(0);
        let contents = toml::to_string_pretty(&default_conf)?;
        conf_file.write_all(contents.as_bytes())?;
        Ok(())
    }
}
